






      subroutine pdbdst( ntree,  nvrtx,  parent, stptr,  stvrtx )
c     ------------------------------------------------------------------
c     pdbdst: This subroutine builds a data structure for accessing each
c             individual tree in a spanning forest from subroutine
c             pdmxst. Vertices are listed in ascending order of indices
c             in each tree list.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             ntree : number of trees in spanning forest.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer ntree , nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             parent(1:nvrtx  ): parent array from pdmxst.
c             stptr (1:ntree+1): pointer to first vertex in a tree.
c             stvrtx(1:nvrtx  ): list of vertices in spanning forest.
c     ------------------------------------------------------------------
      integer parent(1:nvrtx  ),
     +        stptr (1:ntree+1),
     +        stvrtx(1:nvrtx  )

c     ------------------------------------------------------------------
c     Integer variables:
c
c             tree  : tree index.
c             vrtx  : vertex index.
c     ------------------------------------------------------------------
      integer tree  , vrtx

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Initialize pointers.
c     ------------------------------------------------------------------
      do 10 tree=1,ntree
         stptr(tree)=0
 10   continue

c     ------------------------------------------------------------------
c     Count number of vertices in each tree.
c     ------------------------------------------------------------------
      do 20 vrtx=1,nvrtx
         tree=parent(vrtx)
         stptr(tree)=stptr(tree)+1
 20   continue

c     ------------------------------------------------------------------
c     Compute pointers.
c     ------------------------------------------------------------------
      stptr(ntree+1)=nvrtx-stptr(ntree)+1
      do 30 tree=ntree,2,-1
         stptr(tree)=stptr(tree+1)-stptr(tree-1)
 30   continue
      stptr(1)=1

c     ------------------------------------------------------------------
c     Build data structure holding spanning forest.
c     ------------------------------------------------------------------
      do 40 vrtx=1,nvrtx
         tree=parent(vrtx)+1
         stvrtx(stptr(tree))=vrtx
         stptr(tree)=stptr(tree)+1
 40   continue
      return

c     ------------------------------------------------------------------
c     End of subroutine pdbdst.
c     ------------------------------------------------------------------
      end
