






      subroutine pdbkst( ntree,  nedge,  nvrtx,  strv, endv, stptr, 
     +                  stvrtx,  stedg,  rhs )
c     ------------------------------------------------------------------
c     pdbkst: Given a spanning tree defined by a list of vertices in 
c             dfs order and parent edges, it computes the solution the
c             system s'x = b (back substitution).  s is a block diagonal
c             incidence matrix with one linear dependent row per block.
c             The rhs is mapped to each vertex, as is s were a square
c             matrix with a "zero" edge added to the beginning of each 
c             block.  Solution is returned on rhs.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             ntree : number of trees in spanning forest.
c             nedge : number of edges.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer ntree , nedge , nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             strv  (1:  nedge): start vertex in network data structure.
c             stptr (1:    ntree + 1): pointer to first vertex in a tree
c             stvrtx(1:    nvrtx    ): dfs ordering of vertices.
c             stedg (1:    nedge    ): parent edge for each vertex.
c     ------------------------------------------------------------------
      integer
     +        strv  (1:nedge    ),
     +        endv  (1:nedge    ),
     +        stptr (1:    ntree + 1),
     +        stvrtx(1:    nvrtx    ),
     +        stedg (1:    nedge    )

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             rhs   (1:    nvrtx    ): rhs and solution of system.
c     ------------------------------------------------------------------
      double precision
     +        rhs(1:nvrtx)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             edg   : edge index.
c             tree  : tree counter.
c             ptr   : temporary pointer.
c             v1    : vertex index.
c             v2    : adjacent vertex.
c             vrtx  : vertex index.
c     ------------------------------------------------------------------
      integer edg   , tree  , ptr   , v1    , v2    , vrtx

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Process each tree.
c     ------------------------------------------------------------------
      do 20 tree=1,ntree

c        ---------------------------------------------------------------
c        Set variable corresponding to root to zero.
c        ---------------------------------------------------------------
         rhs(stvrtx(stptr(tree)))=0.d0

c        ---------------------------------------------------------------
c        Process remainder of tree.
c        ---------------------------------------------------------------
         do 10 ptr=stptr(tree)+1,stptr(tree+1)-1

c           ------------------------------------------------------------
c           Identify vertices and edge.
c           ------------------------------------------------------------
            vrtx=stvrtx(ptr)
            edg=stedg(vrtx)
            v1=strv(edg)
            v2=endv(edg)

c           ------------------------------------------------------------
c           Solve for current edge.
c           ------------------------------------------------------------
            if(vrtx.eq.v1) then
               rhs(v1)=rhs(v1)+rhs(v2)
            else
               rhs(v2)=-rhs(v2)+rhs(v1)
            endif
 10      continue
 20   continue

c     ------------------------------------------------------------------
c     End of subroutine pdbckst.
c     ------------------------------------------------------------------
      end
