      subroutine pdckfs( b,      cap,     endv,  ierror, nedge,  nvrtx,
     +                   strv, wrke1 )
c     ------------------------------------------------------------------
c     pdckfs : This subroutine checks if the network has enough capacity 
c             for the proposed flow.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             ierror: error code returned (0=no error).
c             nedge : number of arcs.
c             nvrtx : number of nodes.
c     ------------------------------------------------------------------
      integer ierror, nedge, nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             strv  (1:  nedge): start vertex in network data structure.
c             wrke1 (1:  nedge): temporary workspace.
c     ------------------------------------------------------------------
      integer
     +        endv (1:   nedge),
     +        strv (1:   nedge),
     +        wrke1 (1:  nedge)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             b     (1:  nvrtx): array of supply at each node.
c             cap   (1:  nedge): upper bound of flow at each edge
c     ------------------------------------------------------------------
      double precision
     +        b     (1:  nvrtx),
     +        cap   (1:  nedge)

c     ------------------------------------------------------------------
c     Integer variables
c
c             i     : loop counter.
c             mflow : maximum flow supplied.
c             mfreq : maximum flow requested.
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             nactmf: number of edges in active list.
c             ncapmf: number of edges at capacity.
c     ------------------------------------------------------------------
      integer i     , mflow , mfreq , mxfe  , mxfv  , nactmf, ncapmf

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             adem  : overall supply.
c             asup  : overall demand.
c             bi    : capacity at node i.
c     ------------------------------------------------------------------
      double precision adem  , asup  , bi

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      ierror = 0

c     ------------------------------------------------------------------
c     Check if the overall supply is equal to the overall demand.
c     ------------------------------------------------------------------
      asup = 0.0d0
      adem = 0.0d0
      do 10 i = 1, nvrtx
         bi = b(i)
         if (bi .lt. 0.0d0) adem = adem - bi
         if (bi .gt. 0.0d0) asup = asup + bi
 10   continue
      if (asup .ne. adem) then
         ierror = 6
         return
      endif

c     ------------------------------------------------------------------
c     Place all edges in the active list.
c     ------------------------------------------------------------------
      nactmf = 0
      ncapmf = 0
      do 20 i = 1, nedge
         nactmf = nactmf + 1
         wrke1(nactmf) = nedge - i + 1
 20   continue

c     ------------------------------------------------------------------
c     Solve maxflow problem.
c     ------------------------------------------------------------------
      call pdfeas( nvrtx,  nedge,   strv,   endv,   cap,      b, nactmf,
     +             ncapmf, wrke1,  mfreq,  mflow,  mxfv,   mxfe)
      if (mflow .ne. int(asup)) ierror = 7

c     ------------------------------------------------------------------
c     End of subroutine pdckfs.
c     ------------------------------------------------------------------
      end
