






      subroutine pddlot( cost,   endv,  nedge,  ntree,  nvrtx,  stedg,
     +                   stptr,  strv, stvrtx, yopt,   ysol )
c     ------------------------------------------------------------------
c     pddlot: (Orthogonally) project current interior dual solution 
c             onto hyperplane defined by spanning forest.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             nedge : number of edges.
c             ntree : number of trees in spanning forest.
c             nvrtx : number of vertices.
c     ------------------------------------------------------------------
      integer nedge , ntree , nvrtx

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endv  (1:  nedge): end vertex in network data structure
c             stedg (1:    nvrtx    ): parent edge for each vertex.
c             stptr (1:    ntree + 1): pointer to first vertex in a tree
c             strv  (1:  nedge): start vertex in network data structure.
c             stvrtx(1:    nvrtx    ): triangular ordering of vertices.
c     ------------------------------------------------------------------
      integer
     +        endv  (1:    nedge    ),
     +        stedg (1:    nvrtx    ),
     +        stptr (1:    ntree + 1),
     +        strv  (1:    nedge    ),
     +        stvrtx(1:    nvrtx    )

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             cost(1:nedge): cost coefficients.
c             yopt(1:nvrtx): boundary dual solution.
c             ysol(1:nvrtx): interior dual solution.
c     ------------------------------------------------------------------
      double precision
     +        cost(1:nedge),
     +        yopt(1:nvrtx),
     +        ysol(1:nvrtx)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             edg   : edge index
c             ptr   : temporary pointer
c             tree  : tree counter
c             vrtx  : vertex index
c     ------------------------------------------------------------------
      integer edg   , ptr   , tree  , vrtx

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             tmp   : temporary variable.
c     ------------------------------------------------------------------
      double precision
     +        tmp

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Build rhs for non-homogeneous system.
c     ------------------------------------------------------------------
      do 20 tree=1,ntree
         yopt(stvrtx(stptr(tree)))=0.d0
         do 10 ptr=stptr(tree)+1,stptr(tree+1)-1
            vrtx=stvrtx(ptr)
            edg=stedg(vrtx)
            yopt(vrtx)=cost(edg)
 10      continue
 20   continue

c     ------------------------------------------------------------------
c     Compute non-homogeneous solution.
c     ------------------------------------------------------------------
      call pdbkst( ntree,  nedge,  nvrtx,  strv,  endv, stptr,  stvrtx,
     +             stedg,  yopt )
      do 30 tree=1,nedge
 30   continue

c     ------------------------------------------------------------------
c     Compute orthogonal projection for each tree.
c     ------------------------------------------------------------------
      do 60 tree=1,ntree

c     ------------------------------------------------------------------
c     Compute factor for homogeneous solution.
c     ------------------------------------------------------------------
         tmp=0.d0
         do 40 ptr=stptr(tree),stptr(tree+1)-1
            vrtx=stvrtx(ptr)
            tmp=tmp-ysol(vrtx)-yopt(vrtx)
 40      continue
         tmp=tmp/dble(stptr(tree+1)-stptr(tree))

c        ---------------------------------------------------------------
c        Compute projection for current tree.
c        ---------------------------------------------------------------
         do 50 ptr=stptr(tree),stptr(tree+1)-1
            vrtx=stvrtx(ptr)
            yopt(vrtx)=yopt(vrtx)+tmp
 50      continue
 60   continue
      return

c     ------------------------------------------------------------------
c     End of subroutine pddlot.
c     ------------------------------------------------------------------
      end
