



      subroutine pdfdst( outpt,  parent, x )
c     ------------------------------------------------------------------
c     pdfdst: This subroutine returns the canonical element of the set
c             containing  x. Each set is represented by a rooted tree.
c             The nodes of the tree are the elements of the set; the
c             canonical element is the root of the tree.  Each node x
c             has a pointer p(x) to its parent in the tree; the root
c             points to itself.
c
c     Reference:
c     R.E. Tarjan, "Data structures and network algorithms",SIAM,
c     Philadelphia, PA, 1983.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             outpt : canonical element of the set containing x.
c             x     : set element.
c     ------------------------------------------------------------------
      integer outpt , x

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             parent(): parent array for tree representation.
c     ------------------------------------------------------------------
      integer parent(*)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             c     : canonical element.
c             l     : pointer to last element.
c             t     : temporary pointer in search.
c     ------------------------------------------------------------------
      integer c     , l     , t

c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c
c     Find canonical element.
c     ------------------------------------------------------------------
      t=x
 10   if(t.eq.parent(t)) go to 20
      t=parent(t)
      go to 10
 20   continue
c     ------------------------------------------------------------------
c     Canonical element found.
c     ------------------------------------------------------------------
      c=t
      outpt=t
c     ------------------------------------------------------------------
c     Path compression.
c     ------------------------------------------------------------------
      t=x
 30   if(t.eq.parent(t)) go to 40
           l=t
           t=parent(t)
           parent(l)=c
           goto 30
 40   continue
      return

c     ------------------------------------------------------------------
c     End of subroutine pdfdst.
c     ------------------------------------------------------------------
      end
