



      subroutine pdginf( bound , cgstop, dinfo,  dobj,   dof, 
     +                   factor, fcttol, gap,    grho0,  gtolcg,
     +                   gtolmf, huge,   ierror, info,   ioflag,
     +                   iout,   iprcnd, maxit,
     +                   mfdopt, mfstat, miu,    mntlcg, mr, mrho0,
     +                   mxcgit, mxfe,   mxfv,   nbound,  nbuk,
     +                   oldcos, oldtol, opt,    optgap, optmf, 
     +                   optst , pcgcos, pcgit,  pcgres, pduit,  
     +                   pf,     pobj,     pof,    prttyp, rho0,   
     +                   root,   s1fctr,stptol, stpval, tol1,   
     +                   tol2,   tolcg, tolcg0, tolpcg, tolslk, 
     +                   tolsw1, tolsw2, sw1, sw2, zero)
c     ------------------------------------------------------------------
c     pdginf: Gets values of all parameters and statistics from arrays
c             info() and dinfo().
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             info  (1:30): array with integer statistics and parameters.
c     ------------------------------------------------------------------
      integer info(1:30)

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             bfirst: used for building maximum spanning tree.
c             bound : used for computing aditional data structures.
c             cgstop: flags the way PCG stopped.
c             ierror: flags errors in input.
c             ioflag: controls the level of output of summary.
c             iout  : output file unit.
c             iprcnd: preconditioner used.
c             larcs : auxiliary variable.
c             lfirst: auxiliary variable.
c             maxit : maximum number of primal-dual iterations.
c             mfstat: maxflow status.
c             mr    : estimate of maximum number of iterations.
c             mxcgit: maximum number of PCG iterations.
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             na    : number of arcs.
c             nbound: used for computing aditional data structures.
c             nbuk  : number of buckets.
c             nn    : number of nodes.
c             opt   : flags optimal flow.
c             optgap: Duality gap optimality indicator flag.
c             optmf : Maximum flow optimality indicator flag.
c             optst : Spanning tree optimality indicator flag.
c             pcgit : number of PCG iterations performed.
c             pduit : number of primal-dual iterations performed.
c             pf    : iteration level.
c             prttyp: printing level.
c             rc    : flags optimality status.
c             root  : sets beginning of structure.
c             uarcs : number of arcs at the upper bound.
c             ufirst: auxiliary variable.
c     ------------------------------------------------------------------
      integer bfirst, bound , cgstop, ierror, ioflag, iout  , iprcnd,
     +        larcs , lfirst, maxit , mfstat,
     +        mr    , mxcgit, mxfe  , mxfv  , na    , nbound, nbuk  ,
     +        nn    , opt   , optgap, optmf , optst , pcgit , pduit , 
     +        pf    , prttyp, rc    , root  , uarcs , ufirst
      integer i

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             dinfo  (1:35): array with double precision statistics and 
c                            parameters.
c     ------------------------------------------------------------------
      double precision dinfo(1:35)

c     ----------------------------------------------------------------
c     Double precision input/output variables:
c
c             dobj  : dual objective function value.
c             dof   : dual objective function value on tree.
c             ds    : updating parameter for vector s.
c             factor: factor for Newton step.
c             fadd  : auxiliary variable for cost computation.
c             fcttol: factorization zero tolerance.
c             gap   : value of the gap.
c             grho0 : factor for rho0 update.
c             gtolmf: update factor for tol1 and tol2.
c             huge  : maxreal number.
c             maxb  : largest absolute value of b.
c             maxc  : largest absolute value of c.
c             maxu  : largest absolute value of u.
c             mfdopt: maxflow value.
c             minb  : smallest absolute value of b.
c             minc  : smallest absolute value of c.
c             minu  : smallest absolute value of u.
c             miu   : value of interior-point mu parameter.
c             mntlcg: lower bound for CG tolerance.
c             mrho0 : lower bound for rho0.
c             norb  : 1-norm of vector b.
c             norc  : 1-norm of vector c.
c             noru  : 1-norm of vector u.
c             oldcos: value of cosine on previous PCG iteration.
c             oldtol: value of residual on previous PCG iteration.
c             pcgcos: value of cosine on PCG iteration.
c             pcgres: value of residual on PCG iteration.
c             pobj  : objective function value.
c             pof   : primal objective function value on tree.
c             ps    : maximum primal stepsize.
c             rgap  : relative primal-dual gap.
c             rho0  : parameter from IQRD preconditioner.
c             s1fctr: factor for miu.
c             stptol: zero for dual slacks.
c             stpval: largest slack on dual face.
c             sw1   : iteration limit on CG for preconditioner switch.
c             sw2   : another limit for switch.
c             tol1  : lower bound of the tolerance for maxflow.
c             tol2  : upper bound of the tolerance for maxflow.
c             tolcg : tolerance for PCG stopping criterion.
c             tolcg0: guess for tolerance for PCG stopping criterion.
c             tolpcg: initial value for tolcg.
c             weight: tree weight.
c             zero  : value for zero.
c     ------------------------------------------------------------------
      double precision dobj  , dof   , ds, factor,
     +                 fadd  , fcttol, gap   , grho0 , gtolcg,
     +                 gtolmf, huge  , maxb  , maxc  , maxu  ,
     +                 mfdopt, minb  , minc  , minu  , miu   ,
     +                 mntlcg, mrho0 , norb  , norc  , noru  ,
     +                 oldcos, oldtol, pcgcos, pcgres, pobj  ,
     +                 pof   , ps    , rgap  , rho0  , s1fctr,
     +                 stptol, stpval, sw1   , sw2   , tol1  ,
     +                 tol2  , tolcg , tolcg0, tolpcg, tolslk,
     +                 tolsw1, tolsw2, weight, zero
c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
c
c     ------------------------------------------------------------------
c     Integer statistics assignment.
c     ------------------------------------------------------------------
      bound  =  info( 1)
      iprcnd =  info( 2)
      iout   =  info( 3)
      maxit  =  info( 4)
      mxcgit =  info( 5)
      mxfv   =  info( 6)
      mxfe   =  info( 7)
      nbound =  info( 8)
      nbuk   =  info( 9)
      pduit  =  info(10)
      root   =  info(11)
      opt    =  info(12)
      pcgit  =  info(13)
      pf     =  info(14)
      ierror =  info(15)
      cgstop =  info(16)
      mfstat =  info(17)
      ioflag =  info(18)
      prttyp =  info(19)
      optgap =  info(20)
      optmf  =  info(21)
      optst  =  info(22)
      mr     =  info(23)

c     ------------------------------------------------------------------
c     Double precision statistics assignment.
c     ------------------------------------------------------------------
      dobj   = dinfo( 1)
      dof    = dinfo( 2)
      factor = dinfo( 3)
      fcttol = dinfo( 4)
      gap    = dinfo( 5)
      grho0  = dinfo( 6)
      gtolcg = dinfo( 7)
      gtolmf = dinfo( 8)
      mfdopt = dinfo( 9)
      miu    = dinfo(10)
      mntlcg = dinfo(11)
      mrho0  = dinfo(12)
      oldtol = dinfo(13)
      pcgres = dinfo(14)
      pobj   = dinfo(15)
      pof    = dinfo(16)
      rho0   = dinfo(17)
      s1fctr = dinfo(18)
      stptol = dinfo(19)
      stpval = dinfo(20)
      tol1   = dinfo(21)
      tol2   = dinfo(22)
      tolcg  = dinfo(23)
      tolcg0 = dinfo(24)
      tolslk = dinfo(25)
      huge   = dinfo(26)
      tolpcg = dinfo(27)
      oldcos = dinfo(28)
      pcgcos = dinfo(29)
      zero   = dinfo(30)
      tolsw1 = dinfo(31)
      tolsw2 = dinfo(32)
      sw1    = dinfo(33)
      sw2    = dinfo(34)

 
c     ------------------------------------------------------------------
c     End of subroutine pdginf  .
c     ------------------------------------------------------------------
      end
