






      subroutine pdhprm( bfirst, bposit, bucket, endn,   father, first, 
     +                   huge,   infrst, innext, key,    larcs,  lfirst, 
     +                   list,   llink,  lost,   lsets,  mark,   mr, 
     +                   na,     nn,     oufrst, ounext, pred,   rank,
     +                   rlink,  root,   rsets,  rtreth, strn,   teta,  
     +                   trearc, treath, uarcs,  ufirst, weight )
c     ------------------------------------------------------------------
c     pdhprm: This subroutine picks-up a minimum spanning tree from a
c             directed network. It uses a Fibonacci heap data structure
c             to maintain the set of nodes that have not yet been 
c             included in the spanning tree.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c              bfirst: used for building maximum spanning tree.
c              larcs : auxiliary variable.
c              lfirst: auxiliary variable.
c              mr    : estimate of maximum number of iterations.
c              na    : number of arcs.
c              nn    : number of nodes.
c              root  : sets beginning of tree structure.
c     ------------------------------------------------------------------
      integer  bfirst, larcs , mr    , na    , nn, root

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             bposit(1:na): flags arcs according to bounds.
c             bucket(0:mr): stores buckets.
c             endn  (1:na): pointer of network data structure.
c             father(1:nn): marks the father of each node.
c             first (1:nn): first node.
c             infrst(1:nn): points to the first node in in-adcncy list.
c             innext(1:na): points to the last node in in-adcncy list.
c             list  (1:nn): temporary list of nodes.
c             llink (0:nn): part of the node left linked list.
c             lost  (1:nn): number of lost nodes per node.
c             lsets (1:na): list of arcs at lower bound.
c             mark  (1:nn): selects nodes to belong to the tree.
c             oufrst(1:nn): points to the first node in out-adcncy list.
c             ounext(1:na): points to the last node in out-adcncy list.
c             pred  (1:nn): list of predecessors.
c             rank  (1:nn): rank of nodes.
c             rlink (0:nn): part of the node right linked list.
c             rsets (1:na): list of arcs at upper bound.
c             rtreth(1:nn): adjacent to a given node.
c             strn  (1:na): adjacency list of network data structure.
c             trearc(1:nn): arc of tree corresponding to node.
c             treath(1:nn): node that is adjacent to given.
c     ------------------------------------------------------------------
      integer  bposit(1:na), bucket(0:mr), endn  (1:na), father(1:nn),
     +         first (1:nn), infrst(1:nn), innext(1:na), list  (1:nn),
     +         llink (0:nn), lost  (1:nn), lsets (1:na), mark  (1:nn),
     +         oufrst(1:nn), ounext(1:na), pred  (1:nn), rank  (1:nn),
     +         rlink (0:nn), rsets (1:na), rtreth(1:nn), strn  (1:na),
     +         trearc(1:nn), treath(1:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c              key (0:nn): array of key of nodes.
c              teta(1:na): array of teta values of arcs.
c     ------------------------------------------------------------------
      double precision
     +         key (0:nn),
     +         teta(1:na)

c     ------------------------------------------------------------------
c     Integer variables:
c
c              arc   : current arc.
c              enod  : stores a node.
c              larc  : auxiliary variable.
c              lauxrc: variable used for setting up rsets.
c              lfirst: first arc at the lower bound.
c              minkey: minimum key.
c              node  : loop counter.
c              oldmk : variable used for setting up mark.
c              rarc  : auxiliary variable.
c              snod  : stores a node.
c              uarcs : arcs at the upper bound.
c              ufirst: first arc at the upper bound.
c     ------------------------------------------------------------------
      integer  arc   , enod  , larc  , lauxrc, lfirst, minkey, node  ,
     +         oldmk , rarc  , snod  , uarcs , ufirst

c     ------------------------------------------------------------------
c     Double precision variables:
c
c              huge  : very huge number.
c              value : auxiliary variable.
c              weight: weight of the tree.
c     ------------------------------------------------------------------
      double precision huge  , value , weight

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c
c     Initializations.
c     ------------------------------------------------------------------
      do 10 node=1,nn
         key(node)     = huge
         pred(node)    = 0
         first(node)   = 0
         llink(node)   = 0
         rlink(node)   = 0
         rank(node)    = 0
         lost(node)    = 0
         list(node)    = 0
         father(node)  = 0
         mark(node)    = 0
         treath(node)  = 0
         rtreth(node)  = 0
         trearc(node)  = 0
 10   continue
      do 20 node=0,mr
         bucket(node)  = 0
 20   continue
      key(0)        = huge
      llink(0)      = 0
      rlink(0)      = 0
      weight        = 0d0
      bfirst  = 0

c     ------------------------------------------------------------------
c     Define minkey.
c     ------------------------------------------------------------------
      minkey = root

c     ------------------------------------------------------------------
c     Insert nodes in the heap.
c     ------------------------------------------------------------------
      do 30 node=1,nn
         call pdinst( nn,     mr,     node,   pred,   first,  llink,
     +                rlink,  rank,   bucket, key,    minkey )
 30   continue

c     ------------------------------------------------------------------
c     Prim's algorithm.
c     ------------------------------------------------------------------
      node = 1
 40   if (node.eq.nn) goto 90
      node = node + 1

c     ------------------------------------------------------------------
c     Select node minkey to belong to the tree.
c     ------------------------------------------------------------------
      oldmk = minkey
      mark(oldmk) = node-1

c     ------------------------------------------------------------------
c     Delete node minkey from the heap.
c     ------------------------------------------------------------------
      call pddmin( bucket, first,  huge,   key,    llink,  lost,
     +             minkey, mr,     nn,     pred,   rank,   rlink )

c     ------------------------------------------------------------------
c     Decreasing the key of the nodes adjacent to the node oldmk.
c     
c     Incoming arcs:
c     ------------------------------------------------------------------
      arc = infrst(oldmk)
 50   if (arc.eq.0) goto 60
      snod = strn(arc)
      if (key(snod).gt.teta(arc).and.mark(snod).eq.0) then
         trearc(snod) = arc
         value = teta(arc)
         father(snod) = oldmk
         call pddkey( bucket, first , snod  , key   , list  , llink ,
     +                lost  , minkey, mr    , nn    , pred  , rank  ,
     +                rlink , value )
      endif
      arc = innext(arc)
      goto 50

c     ------------------------------------------------------------------
c     Outgoing arcs:
c     ------------------------------------------------------------------
 60   arc = oufrst(oldmk)
 70   if (arc.eq.0) goto 80
      enod = endn(arc)
      if (key(enod).gt.teta(arc).and.mark(enod).eq.0) then
         trearc(enod) = arc
         value = teta(arc)
         father(enod) = oldmk
         call pddkey( bucket, first , enod  , key   , list  , llink ,
     +                lost  , minkey, mr    , nn    , pred  , rank  ,
     +                rlink , value )
      endif
      arc = ounext(arc)
      goto 70

c     ------------------------------------------------------------------
c     Update vectors treath, rtreth, lsets and rsets.
c     ------------------------------------------------------------------
 80   treath(oldmk) = minkey
      rtreth(minkey) = oldmk
      arc  = trearc(minkey)
      if (arc.ne.0) then
         larc = lsets(arc)
         rarc = rsets(arc)
         if (rarc.ne.0) lsets(rarc) = larc
         if (larc.ne.0) rsets(larc) = rarc
         if (arc.eq.lfirst) lfirst=rarc
         if (arc.eq.ufirst) ufirst=rarc
         lsets(arc) = 0
         rsets(arc) = 0
         if (bfirst.eq.0) then
            bfirst         = arc
            lauxrc        = bfirst
         else
            rsets(lauxrc) = arc
            lsets(arc)     = lauxrc
            lauxrc        = arc
         endif
         if (bposit(arc).eq.-1) larcs=larcs-1
         if (bposit(arc).eq. 1) uarcs=uarcs-1
         bposit(arc) = 0
      else
         stop 'disconnected network'
      endif

c     ------------------------------------------------------------------
c     Update tree weight.
c     ------------------------------------------------------------------
      weight = weight + key(minkey)
      goto 40

c     ------------------------------------------------------------------
c     Update vectors treath and rtreth.
c     ------------------------------------------------------------------
 90   treath(minkey) = root
      rtreth(root)  = minkey
      mark(minkey)   = node

c     ------------------------------------------------------------------
c     End of subroutine pdhprm.
c     ------------------------------------------------------------------
      end
