






      subroutine pdiqrd( av4,    diag,   endn,   father, fcttol, iprcnd,
     +                   na,     ndiag,  nn,     rho0,   root,   rtreth,
     +                   strn,   teta,   trearc )
c     ------------------------------------------------------------------
c     pdiqrd: This subroutine constructs the diagonal matrix diag and
c             the vector ndiag wich stores the nondiagonal elements of
c             the matrix f.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             iprcnd: preconditioner used.
c             na    : number of arcs.
c             nn    : number of nodes.
c             root  : sets beginning of structure.
c     ------------------------------------------------------------------
      integer iprcnd, na    , nn    , root

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endn  (1:na): pointer of network data structure.
c             father(1:nn): marks the father of each node.
c             rtreth(1:nn): adjacent to a given node.
c             strn  (1:na): adjacency list of network data structure.
c             trearc(1:nn): arc of tree corresponding to node.
c     ------------------------------------------------------------------
      integer
     +        endn  (1:na),
     +        father(1:nn),
     +        rtreth(1:nn),
     +        strn  (1:na),
     +        trearc(1:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             av4   (1:na): auxiliary vector.
c             diag  (1:nn): diagonal of matrix.
c             ndiag (1:nn): diagonal of preconditioner.
c             teta  (1:na): diagonal theta.
c     ------------------------------------------------------------------

      double precision
     +                 av4   (1:na),
     +                 diag  (1:nn),
     +                 ndiag (1:nn),
     +                 teta  (1:na)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             arc   : current arc.
c             enod  : current element on endn.
c             epred : father of enod.
c             i     : loop counter.
c             node  : another loop counter.
c             pred  : father of node.
c             snod  : current element on strn.
c             spred : father of snode.
c     ------------------------------------------------------------------
      integer arc   , enod  , epred , i    , node  , pred  , snod  ,
     +        spred

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             fcttol: factorization zero tolerance.
c             rho0  : parameter from IQRD preconditioner.
c     ------------------------------------------------------------------
      double precision fcttol, rho0

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             aux1  : auxiliary variable 1.
c             aux2  : auxiliary variable 2.
c             aux3  : auxiliary variable 3.
c             aux4  : auxiliary variable 4.
c             maxav4: maximum element of vector av4.
c             minav4: minimum element of vector av4.
c             rho   : parameter from IQRD preconditioner.
c     ------------------------------------------------------------------
      double precision aux1  , aux2  , aux3  , aux4  , maxav4, minav4,
     +                 rho

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
      do 10 node=1,nn
         diag(node)  = 0d0
         ndiag(node) = 0d0
 10   continue
      minav4=1.0d64
      maxav4=0.0d0
      do 20 arc=1,na
         av4(arc)    = 1d0/teta(arc)
         snod  = strn(arc)
         enod  = endn(arc)
         if (arc.eq.trearc(snod).or.arc.eq.trearc(enod)) then
            maxav4=dmax1(maxav4,av4(arc))
            minav4=dmin1(minav4,av4(arc))
         endif
 20   continue
      if (maxav4 .ne. 0.0d0) then
         rho=rho0*dsqrt(minav4/maxav4)
      else
         rho=rho0
      endif
c     ------------------------------------------------------------------
c     Compute diag.
c     ------------------------------------------------------------------
      if (iprcnd.eq.1) then
         do 30 arc=1,na
            snod  = strn(arc)
            enod  = endn(arc)
            diag(enod)  = diag(enod)+av4(arc)
            diag(snod)  = diag(snod)+av4(arc)
 30      continue
         diag(root) = 0d0
      endif
      if (iprcnd.eq.0) then
         do 40 arc=1,na
            snod  = strn(arc)
            enod  = endn(arc)
            spred = father(snod)
            epred = father(enod)
            if (snod.eq.epred) then
               diag(enod)  =  diag(enod)+av4(arc)
               if (snod.eq.root) goto 40
               ndiag(enod) = ndiag(enod)+av4(arc)
            else
               if (enod.eq.spred) then
                  diag(snod)  =  diag(snod)+av4(arc)
                  if (enod.eq.root) goto 40
                  ndiag(snod) = ndiag(snod)+av4(arc)
               else
                  diag(enod)  = diag(enod)+av4(arc)
                  diag(snod)  = diag(snod)+av4(arc)
               endif
            endif
 40      continue
         diag(root) = 0d0
         do 50 node = 1,root-1
            ndiag(node) = ndiag(node)/diag(node)
 50      continue
         do 60 node = root+1,nn
            ndiag(node) = ndiag(node)/diag(node)
 60      continue
         ndiag(root) = 0d0
      endif
      if (iprcnd.eq.4) then
         do 70 arc = 1,na
            snod = strn(arc)
            enod = endn(arc)
            spred = father(snod)
            epred = father(enod)
            if (arc.eq.trearc(snod).or.arc.eq.trearc(enod)) then
               diag(enod) = diag(enod)+av4(arc)
               diag(snod) = diag(snod)+av4(arc)
            else
               diag(enod) = diag(enod)+rho*av4(arc)
               diag(snod) = diag(snod)+rho*av4(arc)
            endif
            if (snod.eq.epred) then
               if (snod.eq.root) goto 70
               if (arc.ne.trearc(enod)) goto 70
               ndiag(enod) = ndiag(enod)+av4(arc)
            else
               if (enod.eq.spred) then
                  if (enod.eq.root) goto 70
                  if (arc.ne.trearc(snod)) goto 70
                  ndiag(snod) = ndiag(snod)+av4(arc)
               endif
            endif
 70      continue
         diag(root) = 0d0
         ndiag(root) = 0d0
         node = rtreth(root)
         do 80 i = 2, nn
            pred = father(node)
            aux1=diag(node)
            if (aux1.lt.fcttol) then
               aux1 = 1.d50
            endif
            aux2 = ndiag(node)
            aux3 = diag(pred)
       aux4 = aux2/aux1
       ndiag(node) = aux4
       diag(pred) = aux3 - aux2*aux4
       node = rtreth(node)
 80   continue
      endif

c     ------------------------------------------------------------------
c     End of subroutine pdirqd.
c     ------------------------------------------------------------------
      end
