/*-
 * $Id: pdnet.h,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Contents:  General use definitions and macros used in PDNET ANSI C
 *    modules.
 *
 * Authors: Luis Portugal (Univ. Coimbra - Portugal)
 *          Mauricio G. C. Resende (AT&T Labs Research - USA)
 *          Geraldo Veiga (AT&T Labs - USA)
 *          Joaquim Judice (Univ. Coimbra - Portugal)
 *          Joao Patricio (EST Tomar - Portugal)
 *
 * Revision History: 
 *   2001-12-27	Geraldo Veiga
 *       Initial release for ACM distribution.
 *
 */

#if !defined(_H_PDNET)
#define _H_PDNET

/* Include files */
#include <stdlib.h>		/* standard ANSI C */
#include <malloc.h>		/* standard malloc definitions */
#include <string.h>		/* standard string definitions */
#include <stdio.h>		/* standard IO */

/* Constant definitions: */
#ifndef  NULL
#define  NULL 0
#endif

#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif

/* Type definitions: */
#ifndef BOOL
typedef char BOOL;
#endif

/* Macro commands definitions */

/* maximum or minimum independent of type */
#ifndef MAX
#define  MAX(a,b)           ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define  MIN(a,b)           ((a) < (b) ? (a) : (b))
#endif
#ifndef ABS
#define  ABS(a)             ((a) < 0 ? -(a) : (a))
#endif

/* Macro procedure that swaps two entities. */
#define  SWAP(type, a, b)   {type swapx; swapx = a; a = b; b = swapx;}

/* macros to portably convert character case */
#define TOUPPER(c)  ( islower(c) ) ? toupper(c) : (c)
#define TOLOWER(c)  ( isupper(c) ) ? tolower(c) : (c)


/* memory allocation macros */
#define  MALLOC(ptr,type,number) {\
    ptr=(type *) malloc ((size_t)((number) * sizeof(type)));\
    if (ptr == NULL) syserr("malloc failed (" #ptr ")");\
}

#define  REALLOC(ptr,type,number) {\
    ptr=(type *) realloc ((void *)ptr,\
        	 (size_t)((number) * sizeof(type)));\
    if (ptr == NULL) syserr("realloc failed (" #ptr ")" );\
}

#define  CALLOC(ptr,type,number) {\
    ptr=(type *) calloc ((size_t) number, sizeof(type));\
    if (ptr == NULL) syserr("calloc failed (" #ptr ")" );\
}

#define FREE(ptr) {\
    if ((ptr) != NULL) free((void *)(ptr));\
    (ptr) = NULL;\
}

/* Open file */
#define  FOPEN(ptr,name,mode) {\
    {ptr=fopen ((name), (mode));\
    if (ptr == NULL) syserr("Error opening file %s",(name));}\
}

/* Close file */
#define  FCLOSE(ptr) {\
    if (ptr != NULL){\
       if (fclose(ptr) == EOF)\
	  syserr("Error closing file (" #ptr ")" );\
       ptr = NULL;\
    }\
}

/* Binary read */
#define  FREAD(ptr,type,num,file) {\
    if(fread((void *) ptr, sizeof(type), (size_t) num, file) != \
       (size_t) num) usrerr("Error in binary input");\
}

/* Binary write */
#define  FWRITE(ptr,type,num,file) {\
    if(fwrite((void *) ptr, sizeof(type), (size_t) num, file) != \
       (size_t) num) usrerr("Error in binary output");\
}

/* Function Prototypes */

/* Print message and any system message(s) and exit */
extern void syserr (const char *format, ...);

/* Print message and exit */
extern void usrerr (const char *format, ...);

/* Print message and return */
extern void usrmsg (const char *format, ...);

/* Print to stderr and return */
extern int eprintf (const char *format, ...);

#endif /* !_H_PDNET */
