c     $Id: pdout.f,v 1.1.1.1 2002/09/25 18:11:29 ggv Exp $

      subroutine pdwrsl ( c     , dinfo , endn  , info  , l     , 
     +                    na    , nn    , strn  , u     , x     )
c     ------------------------------------------------------------------
c     pdwrsl: Prints summary report.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------  
c     Integer input parameters:
c
c             na : number of arcs.
c             nn : number of nodes.
c     ------------------------------------------------------------------ 
      integer na    , nn

c     ------------------------------------------------------------------ 
c     Integer input arrays:
c             c     (1:na): array of costs at each arc.
c             endn  (1:na): pointer of network data structure.
c             l     (1:na): lower bound of flow at each arc.
c             strn  (1:na): adjacency list of network data structure.
c             u     (1:na): upper bound of flow at each arc.
c             x     (1:na): array with optimal flow at each arc.
c     ------------------------------------------------------------------
      integer
     +        c     (1:na),
     +        endn  (1:na),
     +        l     (1:na),
     +        strn  (1:na),
     +        u     (1:na),
     +        x     (1:na),
     +        info  (1:30)    

c     ------------------------------------------------------------------
c     Double precision input arrays:
c
c             dinfo  (1:    35): array with many statistics.
c     ------------------------------------------------------------------
      double precision dinfo  (1:    35)

c     ------------------------------------------------------------------
c     Integer work variables:
c
c             cgstop: flags the way PCG stopped.
c             i     : loop counter.
c             iprcnd: preconditioner used.
c             itr   : interior-point method iteration count.
c             iout  : output file.
c             mfstat: maxflow status. 
c             mxfe  : edges on optimal flow network. 
c             mxfv  : vertices on optimal flow network.
c             na    : number of arcs.
c             nn    : number of nodes.
c             opt   : flags optimality of solution. 
c             pcgitr: PCG iteration count.
c             pf    : iteration level.
c             prttyp: printing level.
c     ------------------------------------------------------------------
      integer cgstop, i     , iprcnd, itr   , iout  , mfstat, mxfe  , 
     +        mxfv  , opt   , pcgitr, pf    , prttyp

c     ------------------------------------------------------------------
c     Double precision work variables.
c    
c             cgtol : tolerance for PCG stopping criterion.
c             dobj  : dual objective function value on tree.
c             dof   : dual objective function value on tree.
c             gap   : value of the gap.
c             mfdopt: maxflow value.
c             miu   : value of interior-point mu parameter.
c             pcgcos: value of cosine on PCG iteration. 
c             pcgres: value of residual on PCG iteration.
c             pcgtol: value of error on PCG iteration.
c             pobj  : objective function value.
c             pof   : primal objective function value on tree.
c             tol1  : lower bound of the tolerance for maxflow.
c             tol2  : upper bound of the tolerance for maxflow.  
c     ------------------------------------------------------------------ 
      double precision cgtol , dobj  , dof   , gap   , mfdopt, 
     +                 miu   , pcgcos, pcgres, pcgtol, pobj  , 
     +                 pof   , tol1  , tol2       

c     ------------------------------------------------------------------ 
c     Character variables:
c         
c                ccgstp*3 : string corresponding to stopping of CG. 
c                cmfstt*10: string corresponding to maxflow termination.
c                precnd*4 : string corresponding to preconditioner.
c     ------------------------------------------------------------------
      character  ccgstp*3,
     +           cmfstt*10, 
     +           precnd*4                          
     
c     ------------------------------------------------------------------
c     Start of executable portion of subroutine.
c     ------------------------------------------------------------------
      if (info(19) .eq. 0) return
      itr    =  info(10)
      pcgitr =  info(13)
      gap    = dinfo( 5)
      pcgtol = dinfo(27)
      cgtol  = dinfo( 7)
      mfdopt = dinfo( 9)                 
      iprcnd =  info( 2)
      mxfv   =  info( 6)
      mxfe   =  info( 7)
      cgstop =  info(16)
      mfstat =  info(17)
      miu    = dinfo(10)             
      pcgcos = dinfo(29)
      pcgres = dinfo(14)
      dof    = dinfo( 2)
      pof    = dinfo(16)
      pf     =  info(14)
      opt    =  info(12)
      tol1   = dinfo(21)
      tol2   = dinfo(22)         
      pobj   = dinfo(15)
      dobj   = dinfo( 1)
      prttyp =  info(19)
      iout   =  info( 3)
      if (cgstop .eq. 0) then
         ccgstp = 'MAX'
      else
         if (cgstop .eq. 1) then
            ccgstp = 'NOR'
         else
            if (cgstop .eq. 2) then
               ccgstp = 'COS'
            else
               ccgstp ='???'
            endif
         endif
      endif
      if (mfstat .eq. 0) then
         cmfstt = 'NONE'
      else
         if (mfstat .eq. 1) then
            cmfstt = 'NONOPTIMAL'
         else
            if (mfstat .eq. 2) then
               cmfstt = 'OPTIMAL'
            else
               cmfstt = '?'
            endif
         endif
      endif
      if (iprcnd.eq.1) then
         precnd='DIAG'
      else
         if (iprcnd.eq.3) then
            precnd='SPAN'
         else
            if (iprcnd.eq.2) then
               precnd='IQRD'
            else
               if (iprcnd.eq.4) then
                  precnd='DMST'
               else                                              
                  precnd='?'
               endif
            endif
         endif
      endif
      
      write (iout,900)
 900  format('----------------------------------------',
     +     '----------------------------------------')
      if (prttyp .ge. 2) then
         write (iout,902) itr,miu
 902     format('  CONV>     itr: ',i14,'      miu: ',e14.8,/)
         write (iout,901) pobj,dobj,gap
 901     format('  COST>    pobj: ',e14.8,'     dobj: ',e14.8,
     +        '  pd-gap: ',e14.8,/)
         write (iout,903) precnd,ccgstp,pcgitr,pcgtol,pcgres,cgtol,
     +        pcgcos
 903     format('    CG>  precnd: ',a4,7x,'      cgstop: ',a3,3x,
     +        '          pcgitr: ',i14,/,
     +        '    CG>  pcgtol: ',e14.8,'   pcgres: ',e14.8,/,
     +        '    CG>   cgtol: ',e14.8,'   pcgcos: ',e14.8,/)
         if (pf.eq.1 .and. opt.eq.1) then
            write (iout,904)
 904        format('  TREE> pb-feas: TRUE             pb-opt: TRUE')
         endif
         if (pf.eq.1 .and. opt.eq.0) then
            write (iout,905)
 905        format('  TREE> pb-feas: TRUE             pb-opt: FALSE')
         endif
         if (pf.eq.0) then
            write (iout,906)
 906        format('  TREE> pb-feas: FALSE')
         endif
      else
         write (iout,902) itr,miu
         if (pf.eq.1 .and. opt.eq.1) then
            write (iout,904)
         endif
         if (pf.eq.1 .and. opt.eq.0) then
            write (iout,905)
         endif
         if (pf.eq.0) then
            write (iout,906)
         endif
      endif
      
      if (pf.eq.1) then
         write (iout,907) pof,dof
 907     format('  TREE>     pof: ',e14.8,'      dof: ',e14.8)
      endif
      write (iout,911)
      if (miu.lt.1.0d0) then
         write (iout,908) mxfv,mxfe
 908     format(' MFLOW>    mxfv: ',i14,'     mxfe: ',i14)
      endif
      if (prttyp .ge. 2) then
         if (miu.lt.1.0d0) then
            write (iout,909) tol1,tol2,mfdopt
 909        format(' MFLOW> pdlotol: ',e14.8,'  pdhitol: ',e14.8,
     +           '    dobj: ',e14.8)
         endif
         write (iout,910) cmfstt
 910     format(' MFLOW>  mfstat: ',a10)
      endif
      if (miu.lt.1.0d0) then
         write (iout,911)
 911     format(' ')
      endif
      write (iout,900)
      if (prttyp.le.2) return
      write (iout,915)
 915  format (' PDNET 1.0:  Primal-feasible,',
     +     ' dual-infeasible interior point network flow method',/,
     +     ' OPTIMAL primal flow',//,
     +     '                from        to       arc     lower',
     +     '     upper       arc',/,
     +     '       arc      node      node      cost     bound',
     +     '     bound      flow',/)
      do 100 i = 1, na
        if (x(i).gt.0) then
            write (iout,950) i,strn(i),endn(i),c(i),l(i),u(i)+l(i),
     +          x(i)+l(i)
 950        format(7i10)
        endif
 100  continue
      write (iout,900)
      return                                                            

c     ------------------------------------------------------------------
c     End of executable portion of subroutine pdwrsl.
c     ------------------------------------------------------------------
      end                                                               
