






      subroutine pdpdat( b,      bound,  c,      huge,   maxb,   maxc,
     +                   maxu,   minb,   minc,   minu,   na,     nn,
     +                   norb,   norc,   noru,   u,      upfrst, upnext
     +                 )
c     ------------------------------------------------------------------
c     pdpdat: This subroutine finds the data characteristics of the 
c             problem.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c               bound : number of bounded arcs.
c               na    : number of arcs.
c               nn    : number of nodes.
c               upfrst: next unbounded arc to be considered.
c     ------------------------------------------------------------------
      integer   bound , na    , nn    , upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c               upnext(1:na): next arc with positive upper bound.
c     ------------------------------------------------------------------
      integer   upnext(1:na)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c               b(1:nn): array of capacities at each node.
c               c(1:na): array of costs at each arc.
c               u(1:na): upper bound of flow at each arc.
c     ------------------------------------------------------------------
      double precision
     +          b(1:nn),
     +          c(1:na),
     +          u(1:na)

c
c     ------------------------------------------------------------------
c     Integer variables:
c
c               arc   : current arc.
c               i     : loop counter.
c               node  : another loop counter.
c     ------------------------------------------------------------------
      integer   arc   , i     ,  node

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c               huge  : maxreal number.
c               maxb  : largest absolute value of b.
c               maxc  : largest absolute value of c.
c               maxu  : largest absolute value of u.
c               minb  : smallest absolute value of b.
c               minc  : smallest absolute value of c.
c               minu  : smallest absolute value of u.
c               norb  : 1-norm of vector b.
c               norc  : 1-norm of vector c.
c               noru  : 1-norm of vector u.
c     ------------------------------------------------------------------
      double precision    huge  , maxb  , maxc  , maxu  , minb  ,
     +                    minc  , minu  , norb  , norc  , noru

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
      maxc = -huge
      maxu = -huge
      maxb = -huge
      minc =  huge
      minu =  huge
      minb =  huge
      norc = 0d0
      noru = 0d0
      norb = 0d0
      do 10 arc=1,na
         maxc = dmax1(maxc,c(arc))
         minc = dmin1(minc,c(arc))
         norc = norc+dabs(c(arc))
 10   continue
      arc = upfrst
      do 20 i=1,bound
         maxu = dmax1(maxu,u(arc))
         minu = dmin1(minu,u(arc))
         noru = noru+dabs(u(arc))
         arc = upnext(arc)
 20   continue
      do 30 node=1,nn
         maxb = dmax1(maxb,b(node))
         minb = dmin1(minb,b(node))
         norb = norb+dabs(b(node))
 30   continue

c     ------------------------------------------------------------------
c     End of subroutine pdpdat.
c     ------------------------------------------------------------------
      end
