







      subroutine pdprtb( b,      bound,  c,      db,     dc,     du,
     +                   endn,   na,     nn,     s,      strn,   u, 
     +                   upfrst, upnext, w,      x,      z )
c     ------------------------------------------------------------------
c     pdprtb: This subroutine perturbs and transforms the data to
c             double precision.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables.
c
c                   bound : number of bounded arcs.
c                   na    : number of arcs.
c                   nn    : number of nodes.
c                   upfrst: next unbounded arc to be considered.
c     ------------------------------------------------------------------
      integer       bound , na    , nn    , upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             b     (1:nn): array of capacities at each node.
c             c     (1:na): array of costs at each arc.
c             endn  (1:na): pointer of network data structure.
c             strn  (1:na): adjacency list of network data structure.
c             u     (1:na): upper bound of flow at each arc.
c             upnext(1:na): next arc with positive upper bound.
c     ------------------------------------------------------------------
      integer
     +        b     (1:nn),
     +        c     (1:na),
     +        endn  (1:na),
     +        strn  (1:na),
     +        u     (1:na),
     +        upnext(1:na)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             db    (1:nn): double precision copy of int vector b.
c             dc    (1:na): double precision copy of int vector c.
c             du    (1:na): double precision copy of int vector u.
c             s     (1:na): dual iterate s.
c             w     (1:na): dual iterate w.
c             x     (1:na): primal iterate x.
c             z     (1:na): primal iterate z.
c     ------------------------------------------------------------------
      double precision
     +              db    (1:nn),
     +              dc    (1:na),
     +              du    (1:na),
     +              s     (1:na),
     +              w     (1:na),
     +              x     (1:na),
     +              z     (1:na)

c     ------------------------------------------------------------------
c     Integer variables.
c
c                   arc   : current arc.
c                   enod  : current element on endn.
c                   i     : loop counter.
c                   node  : another loop counter.
c                   snod  : current element on strn.
c     ------------------------------------------------------------------
      integer       arc   , enod  , i     , node  , snod

c     ------------------------------------------------------------------
c     Start of executable section of the subroutine.
c     ------------------------------------------------------------------
      do 10 arc=1,na
         x(arc) = 0.0d0
         s(arc) = 0.0d0
         z(arc) = 0.0d0
         w(arc) = 0.0d0
 10   continue
      do 20 node=1,nn
         db(node) = dble(b(node))
 20   continue
      do 30 arc=1,na
         snod = strn(arc)
         enod = endn(arc)
         db(snod) = db(snod)+x(arc)
         db(enod) = db(enod)-x(arc)
         dc(arc) = dble(c(arc))+z(arc)-w(arc)
 30   continue
      arc = upfrst
      do 40 i=1,bound
         du(arc)  = dble(u(arc))+x(arc)+s(arc)
         arc = upnext(arc)
 40   continue

c     ------------------------------------------------------------------
c     End of subroutine pdprtb.
c     ------------------------------------------------------------------
      end
