






      subroutine pdsout( cgstop, cgtol,  dobj,   dof,    gap,    iout,
     +                   iprcnd, itr,    mfdopt, mfstat, miu,    mxfe,
     +                   mxfv,   na,     nn,     opt,    pcgcos, pcgitr,
     +                   pcgres, pcgtol, pf,     pobj,   pof,    prttyp,
     +                   tol1,   tol2,   x )
c     ------------------------------------------------------------------
c     pdsout: This subroutine prints summary report at the end of each
c             iteration.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             cgstop: flags the way PCG stopped.
c             iout  : output file unit.
c             iprcnd: preconditioner used.
c             itr   : number of primal-dual iterations performed.
c             mfstat: maxflow status.
c             mxfe  : edges on optimal flow network.
c             mxfv  : vertices on optimal flow network.
c             na    : number of arcs.
c             nn    : number of nodes.
c             opt   : flags optimal flow.
c             pcgitr: number of PCG iterations performed.
c             pf    : iteration level.
c             prttyp: printing level.
c     ------------------------------------------------------------------
      integer cgstop, iout  , iprcnd, itr   , mfstat,
     +        mxfe  , mxfv  , na    , nn    , opt   , pcgitr, pf    ,
     +        prttyp

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             x(1:na): array with optimal flow at each arc.
c     ------------------------------------------------------------------
      integer x(1:na)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             i     : loop counter.
c     ------------------------------------------------------------------
      integer i

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             cgtol : value of cosine on previous PCG iteration.
c             dobj  : dual objective function value.
c             dof   : dual objective function value on tree.
c             gap   : value of the gap.
c             mfdopt: maxflow value.
c             miu   : value of interior-point mu parameter.
c             pcgcos: value of cosine on PCG iteration.
c             pcgres: value of residual on PCG iteration
c             pcgtol: value of residual on previous PCG iteration.
c             pobj  : objective function value.
c             pof   : primal objective function value on tree.
c             tol1  : lower bound of the tolerance for maxflow.
c             tol2  : upper bound of the tolerance for maxflow.
c     ------------------------------------------------------------------
      double precision cgtol , dobj  , dof   , gap   , mfdopt,
     +                 miu   , pcgcos, pcgres, pcgtol, pobj  ,
     +                 pof   , tol1  , tol2

c     ------------------------------------------------------------------
c     String type variables:
c
c             ccgstp: type of stopping on CG.
c             cmfstt: type of optimality state.
c             precnd: type of preconditioner.
c     ------------------------------------------------------------------
      character
     +          ccgstp*3,
     +          cmfstt*10,
     +          precnd*4

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c     ------------------------------------------------------------------
      if (prttyp .eq. 0) return
      if (cgstop .eq. 0) then
         ccgstp = 'MAX'
      else
         if (cgstop .eq. 1) then
            ccgstp = 'RESIDUAL'
         else
            if (cgstop .eq. 2) then
               ccgstp = 'COSINE'
            else
               ccgstp ='??????'
            endif
         endif
      endif
      if (mfstat .eq. 0) then
         cmfstt = 'INACTIVE'
      else
         if (mfstat .eq. 1) then
            cmfstt = 'SUBOPTIMAL'
         else
            if (mfstat .eq. 2) then
               cmfstt = 'OPTIMAL'
            else
               cmfstt = '?'
            endif
         endif
      endif
      if (iprcnd.eq.1) then
         precnd='DIAGONAL'
      else
         if (iprcnd.eq.3) then
            precnd='SPANNING TREE'
         else
            if (iprcnd.eq.2) then
               precnd='IQRD'
            else
               if (iprcnd.eq.4) then
                  precnd='INCOMPLETE CHOLESKY'
               else
                  precnd='?'
               endif
            endif
         endif
      endif

      write (iout,899)
899   format('----------------------------------------',
     +        '----------------------------------------')

      if (prttyp .ge. 2) then
         write (iout,902) itr,miu
 902     format('  CONV>     itr: ',i14,'       mu: ',e14.8,/)
         write (iout,901) pobj,dobj,gap
 901     format('  COST>    pobj: ',e14.8,'     dobj: ',e14.8,
     +        '  pd-gap: ',e14.8,/)
         write (iout,903) precnd,ccgstp,pcgitr,pcgtol,pcgres,cgtol,
     +        pcgcos
 903     format('    CG>  precnd: ',a4,7x,'      cgstop: ',a4,3x,
     +        '         pcgitr: ',i14,/,
     +        '    CG>  pcgtol: ',e14.8,'   pcgres: ',e14.8,/,
     +        '    CG>   cgtol: ',e14.8,'   pcgcos: ',e14.8,/)
         if (pf.eq.1 .and. opt.eq.1) then
            write (iout,904)
 904        format('  TREE> pb-feas: TRUE             pb-opt: TRUE')
         endif
         if (pf.eq.1 .and. opt.eq.0) then
            write (iout,905)
 905        format('  TREE> pb-feas: TRUE             pb-opt: FALSE')
         endif
         if (pf.eq.0) then
            write (iout,906)
 906        format('  TREE> pb-feas: FALSE')
         endif
      else
         write (iout,902) itr,miu
         if (pf.eq.1 .and. opt.eq.1) then
            write (iout,904)
         endif
         if (pf.eq.1 .and. opt.eq.0) then
            write (iout,905)
         endif
         if (pf.eq.0) then
            write (iout,906)
         endif
      endif
      if (pf.eq.1) then
         write (iout,907) pof,dof
 907     format('  TREE>    pobj: ',e14.8,'     dobj: ',e14.8)
      endif
      write (iout,911)

      if (miu.lt.1.0d0) then
         if (prttyp .ge. 2) then
              write (iout,908) mxfv,mxfe
 908          format(' MFLOW>    mxfv: ',i14,'     mxfe: ',i14)
         endif
      endif
      if (prttyp .ge. 2) then
         if (miu.lt.1.0d0) then
            write (iout,909) tol1,tol2,mfdopt
909         format(' MFLOW> pdlotol: ',e14.8,'  pdhitol: ',e14.8,
     +             '    dobj: ',e14.8)
         endif
      endif
      write (iout,910) cmfstt
 910  format(' MFLOW>  mfstat: ',a10)
      if (miu.lt.1.0d0) then
         write (iout,911)
 911     format(' ')
      endif
      if (prttyp .eq. 3) then
         open (14, file = 'solution.dat', status = 'unknown',
     +        access = 'sequential', form = 'formatted')
         do 100 i = 1, na
            write (14,*) x(i)
 100     continue
         close(14)
      endif
      return

c     ------------------------------------------------------------------
c     End of subroutine pdsout.
c     ------------------------------------------------------------------
      end
