






      subroutine pdsviq( b,      diag,   father, iprcnd, na,     ndiag,
     +                   nn,     root,   rtreth, teta,   trearc, treath,
     +                   x )
c     ------------------------------------------------------------------
c     pdsviq: This subroutine solves a system with the IQRD matrix.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             iprcnd: preconditioner used.
c             na    : number of arcs.
c             nn    : number of nodes.
c             root  : sets beginning of structure.
c     ------------------------------------------------------------------
      integer iprcnd, na    , nn    , root

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             father(1:nn): marks the father of each node.
c             rtreth(1:nn): adjacent to a given node.
c             trearc(1:nn): arc of tree corresponding to node.
c             treath(1:nn): node that is adjacent to given.
c     ------------------------------------------------------------------
      integer
     +        father(1:nn),
     +        rtreth(1:nn),
     +        trearc(1:nn),
     +        treath(1:nn)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             b    (1:nn): array of capacities at each node.
c             diag (1:nn): diagonal of matrix.
c             ndiag(1:nn): diagonal of preconditioner.
c             teta (1:na): diagonal theta.
c             x    (1:nn): primal iteration x.
c     ------------------------------------------------------------------
      double precision
     +                 b    (1:nn),
     +                 diag (1:nn),
     +                 ndiag(1:nn),
     +                 teta (1:na),
     +                 x    (1:nn)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             i     : loop counter.
c             node  : another loop counter.
c             pred  : father of node.
c     ------------------------------------------------------------------
      integer i     , node  , pred

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c
c     Initializing the vector x.
c     ------------------------------------------------------------------
      do 10 node=1,root-1
         x(node) = b(node)
 10   continue
      do 20 node=root+1,nn
         x(node) = b(node)
 20   continue
      x(root) = 0d0

c     ------------------------------------------------------------------
c     Reverse treath traversal of the spanning tree.
c     ------------------------------------------------------------------
      if (iprcnd.eq.3) then
         node = rtreth(root)
         do 30 i=2,nn
            pred = father(node)
            x(pred) = x(pred)+x(node)
            node = rtreth(node)
 30      continue
      endif
      if (iprcnd.eq.0.or.iprcnd.eq.4) then
         node = rtreth(root)
         do 40 i=2,nn
            pred = father(node)
            x(pred) = x(pred)+x(node)*ndiag(node)
            node = rtreth(node)
 40      continue
      endif
      x(root) = 0d0

c     ------------------------------------------------------------------
c     Dividing x by diag.
c     ------------------------------------------------------------------
      if (iprcnd.eq.0.or.iprcnd.eq.1.or.iprcnd.eq.4) then
         do 50 node=1,root-1
            x(node) = x(node)/diag(node)
 50      continue
         do 60 node=root+1,nn
            x(node) = x(node)/diag(node)
 60      continue
      endif
      if (iprcnd.eq.3) then
         do 70 node=1,root-1
            x(node) = x(node)*teta(trearc(node))
 70      continue
         do 80 node=root+1,nn
            x(node) = x(node)*teta(trearc(node))
 80      continue
      endif

c     ------------------------------------------------------------------
c     Treath traversal of the spanning tree.
c     ------------------------------------------------------------------
      if (iprcnd.eq.3) then
         node = treath(root)
         do 90 i=2,nn
            pred = father(node)
            x(node) = x(node)+x(pred)
            node = treath(node)
 90      continue
      endif
      if (iprcnd.eq.0.or.iprcnd.eq.4) then
         node = treath(root)
         do 100 i=2,nn
            pred = father(node)
            x(node) = x(node)+x(pred)*ndiag(node)
            node = treath(node)
 100     continue
      endif

c     ------------------------------------------------------------------
c     End of subroutine pdsviq.
c     ------------------------------------------------------------------
      end
