






      subroutine pdupsl( av1,    av2,    av3,    av4,    av5,    av6,
     +                   av7,    av8,    b,      bound,  ds,     endn,
     +                   factor, gap,    huge,   miu,    na,     nbound,
     +                   nn,     npfrst, ps,     root,   s,      sd,
     +                   strn,   teta,   tolpcg, u,      upfrst, upnext, 
     +                   w,      wd,     x,      xd,     y,      yd,
     +                   z,      zd )
c     ------------------------------------------------------------------
c     pdupsl: This subroutine updates the primal and dual solutions in
c             the network primal-dual interior-point algorithm.
c             Furthermore, this subroutine updates the parameter miu.
c     ------------------------------------------------------------------
      implicit none

c     ------------------------------------------------------------------
c     Integer input/output variables:
c
c             bound : used for computing aditional data structures.
c             na    : number of arcs.
c             nbound: number of bounds.
c             nn    : number of nodes.
c             npfrst: next bounded arc to be considered.
c             root  : sets beginning of structure.
c             upfrst: next unbounded arc to be considered.
c     ------------------------------------------------------------------
      integer bound , na    , nbound, nn    , npfrst, root  , upfrst

c     ------------------------------------------------------------------
c     Integer input/output arrays:
c
c             endn  (1:na): pointer of network data structure.
c             strn  (1:na): adjacency list of network data structure.
c             upnext(1:na): next arc with positive upper bound.
c     ------------------------------------------------------------------
      integer
     +        endn  (1:na),
     +        strn  (1:na),
     +        upnext(1:na)

c     ------------------------------------------------------------------
c     Double precision input/output arrays:
c
c             av1  (1:na): auxiliary vector 1.
c             av2  (1:nn): auxiliary vector 2.
c             av3  (1:na): auxiliary vector 3.
c             av4  (1:na): auxiliary vector 4.
c             av5  (1:na): auxiliary vector 5.
c             av6  (1:na): auxiliary vector 6.
c             av7  (1:na): auxiliary vector 7.
c             av8  (1:na): auxiliary vector 8.
c             b    (1:nn): array of capacities at each node.
c             s    (1:na): dual variable s.
c             sd   (1:na): direction for s.
c             teta (1:na): diagonal matrix theta.
c             u    (1:na): upper bound of flow at each arc.
c             w    (1:na): primal variable w.
c             wd   (1:na): direction for w.
c             x    (1:na): primal variable x.
c             xd   (1:na): direction for x.
c             y    (1:nn): dual variable y.
c             yd   (1:nn): direction for y.
c             z    (1:na): primal variable z.
c             zd   (1:na): direction for z.
c     ------------------------------------------------------------------
      double precision
     +                 av1  (1:na),
     +                 av2  (1:nn),
     +                 av3  (1:na),
     +                 av4  (1:na),
     +                 av5  (1:na),
     +                 av6  (1:na),
     +                 av7  (1:na),
     +                 av8  (1:na),
     +                 b    (1:nn),
     +                 s    (1:na),
     +                 sd   (1:na),
     +                 teta (1:na),
     +                 u    (1:na),
     +                 w    (1:na),
     +                 wd   (1:na),
     +                 x    (1:na),
     +                 xd   (1:na),
     +                 y    (1:nn),
     +                 yd   (1:nn),
     +                 z    (1:na),
     +                 zd   (1:na)

c     ------------------------------------------------------------------
c     Integer variables:
c
c             arc   : current arc.
c             enod  : current element on endn.
c             i     : loop counter.
c             node  : another loop counter.
c             snod  :  current element on strn.
c     ------------------------------------------------------------------
      integer arc   , enod  , i     , node  , snod

c     ------------------------------------------------------------------
c     Double precision input/output variables:
c
c             factor: factor for lambda_max update.
c             gap   : value of the gap.
c             huge  : maxreal number.
c             miu   : value of interior-point mu parameter.
c             tolpcg: initial value for tolcg.
c     ------------------------------------------------------------------
      double precision factor, gap   , huge  , miu   , tolpcg

c     ------------------------------------------------------------------
c     Double precision variables:
c
c             aux1  : auxiliary variable.
c             ds    : dual step.
c             oldps : old dual step.
c             ps    : primal step.
c     ------------------------------------------------------------------
      double precision aux1  , ds    , oldps , ps

c     ------------------------------------------------------------------
c     Start of executable section of subroutine.
c
c     Initializations.
c     ------------------------------------------------------------------
      do 10 node=1,nn
         av2(node) = b(node)
 10   continue
      gap = 0d0
      ps = huge
      ds = huge

c     ------------------------------------------------------------------
c     Compute Newton's direction xd, sd, zd and wd and maximum primal 
c     and dual stepsizes ps and ds.
c
c     Arcs without upper bound.
c     ------------------------------------------------------------------
      arc = npfrst
      do 20 i=1,nbound

c        ---------------------------------------------------------------
c        Compute working parameters.
c        ---------------------------------------------------------------
         snod = strn(arc)
         enod = endn(arc)
         aux1 = -yd(snod)+yd(enod)

c        ---------------------------------------------------------------
c        Compute the Newton's direction.
c        ---------------------------------------------------------------
         xd(arc)  = aux1/teta(arc)+xd(arc)
         zd(arc)  = zd(arc)-av1(arc)*xd(arc)-z(arc)

c        ---------------------------------------------------------------
c        Compute the primal and dual steps.
c        ---------------------------------------------------------------
         if (xd(arc).lt.0.) then
            aux1 = -x(arc)/xd(arc)
            ps = dmin1(ps,aux1)
         endif
         if (zd(arc).lt.0.) then
            aux1 = -z(arc)/zd(arc)
            ds = dmin1(ds,aux1)
         endif

c        ---------------------------------------------------------------
c        Compute the next arc.
c        ---------------------------------------------------------------
         arc = upnext(arc)
 20   continue

c     ------------------------------------------------------------------
c     Arcs with upper bound.
c     ------------------------------------------------------------------
      arc = upfrst
      do 30 i=1,bound

c        ---------------------------------------------------------------
c        Compute working parameters.
c        ---------------------------------------------------------------
         snod = strn(arc)
         enod = endn(arc)
         aux1 = -yd(snod)+yd(enod)

c        ---------------------------------------------------------------
c        Compute Newton's direction.
c        ---------------------------------------------------------------
         xd(arc)  = aux1/teta(arc)+xd(arc)
         sd(arc)  = u(arc)-x(arc)-s(arc)-xd(arc)
         zd(arc)  = zd(arc)-av1(arc)*xd(arc)-z(arc)
         wd(arc)  = wd(arc)-av3(arc)*sd(arc)-w(arc)

c        ---------------------------------------------------------------
c        Compute primal and dual steps.
c        ---------------------------------------------------------------
         if (xd(arc).lt.0.) then
            aux1 = -x(arc)/xd(arc)
            ps = dmin1(ps,aux1)
         endif
         if (sd(arc).lt.0.) then
            aux1 = -s(arc)/sd(arc)
            ps = dmin1(ps,aux1)
         endif
         if (zd(arc).lt.0.) then
            aux1 = -z(arc)/zd(arc)
            ds = dmin1(ds,aux1)
         endif
         if (wd(arc).lt.0.) then
            aux1 = -w(arc)/wd(arc)
            ds = dmin1(ds,aux1)
         endif

c        ---------------------------------------------------------------
c        Compute the next arc.
c        ---------------------------------------------------------------
         arc = upnext(arc)
 30   continue

c     ------------------------------------------------------------------
c     Multiply primal and dual steps by factor.
c     ------------------------------------------------------------------
      ps = ps*factor
      ds = ds*factor
      do 40 node=1,nn
         y(node) = y(node)+ds*yd(node)
 40   continue

c     ------------------------------------------------------------------
c     Set y(root) equal to zero.
c     ------------------------------------------------------------------
      y(root) = 0d0

c     ------------------------------------------------------------------
c     Update vectors x, w, z, and s.  Compute the duality gap.
c
c     Arcs with upper bound
c     ------------------------------------------------------------------
      arc = upfrst
      do 50 i=1,bound
c        ---------------------------------------------------------------
c        Compute working parameters.
c        ---------------------------------------------------------------
         snod = strn(arc)
         enod = endn(arc)

c        ---------------------------------------------------------------
c        Update x, w, z, and s.
c        ---------------------------------------------------------------
         x(arc) = x(arc)+ps*xd(arc)
         s(arc) = s(arc)+ps*sd(arc)
         z(arc) = z(arc)+ds*zd(arc)
         w(arc) = w(arc)+ds*wd(arc)

c        ---------------------------------------------------------------
c        Compute the duality gap.
c        ---------------------------------------------------------------
         gap  = gap+x(arc)*z(arc)+s(arc)*w(arc)

c        ---------------------------------------------------------------
c        Compute auxilliary vector av2.
c        ---------------------------------------------------------------
         av2(snod) = av2(snod)+x(arc)
         av2(enod) = av2(enod)-x(arc)
         arc = upnext(arc)
 50   continue

c     ------------------------------------------------------------------
c     Arcs without upper bound.
c     ------------------------------------------------------------------
      arc = npfrst
      do 60 i=1,nbound
c        ---------------------------------------------------------------
c        Compute working parameters.
c        ---------------------------------------------------------------
         snod = strn(arc)
         enod = endn(arc)

c        ---------------------------------------------------------------
c        Update x, w, z, and s.
c        ---------------------------------------------------------------
         x(arc) = x(arc)+ps*xd(arc)
         z(arc) = z(arc)+ds*zd(arc)

c        ---------------------------------------------------------------
c        Compute the duality gap.
c        ---------------------------------------------------------------
         gap  = gap+x(arc)*z(arc)

c        ---------------------------------------------------------------
c        Compute auxilliary vector av2.
c        ---------------------------------------------------------------
         av2(snod) = av2(snod)+x(arc)
         av2(enod) = av2(enod)-x(arc)
         arc = upnext(arc)
 60   continue

c     ------------------------------------------------------------------
c     Set av2(root) equal to zero.
c     ------------------------------------------------------------------
      av2(root) = 0d0

c     ------------------------------------------------------------------
c     Compute the parameter miu and tolerance for conjugate gradients.
c     ------------------------------------------------------------------
      tolpcg    = 0d0
      do 70 node=1,nn
         tolpcg = tolpcg+dabs(av2(node))
 70   continue
      miu    = gap/dble(5*na)
      tolpcg = tolpcg/1d1
      write (77, *) 'x:',(x(i),i=1,na)

c     ------------------------------------------------------------------
c     End of subroutine pdupsl.
c     ------------------------------------------------------------------
      end
